/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#ifndef MHChart_h
#define MHChart_h

#include "MH.h"
#include "MHChartSettings.h"
#include "MHUtil.h"
#include "MHPoint.h"
#include "MHString.h"
#include "MHVector.h"

class MHDC;



/**
*  MHChart is a base class for charts objects.
*  It must be subclassed to be used.
*/
class MHChart : public MH {
public:
    enum COLORS {
                            BLUE = 0,
                            RED,
                            GREEN,
                            MAGENTA,
                            YELLOW,
                            CYAN,
                            BLACK,
                            GRAY,
                            WHITE,
                            END_COLORS,
    };

    enum VALUES {
                            MAX_DATA             = 50000,
                            MAX_VALUE            = 2100000000,
                            MAX_SERIES           = 6,
                            MIN_CANVAS_HEIGHT    = 50,
                            MIN_CANVAS_WIDTH     = 50,
                            MIN_DATA             = 1,
                            NULL_DATA            = 0,
                            SMALL_TICK_LINE_SIZE = 5,
                            XBORDER              = 2,
                            YBORDER              = 7,
                            XOFFSET              = 3,
                            YOFFSET              = 1,
                            LEFT                 = 0,
                            RIGHT                = 1,
                            DEFAULT_LINE_WIDTH   = 3,
    };

    friend class            MHDC;

                            MHChart ();
    virtual                 ~MHChart ();

    const char*             Class () {return "MHChart";}
    void                    Clear ();
    virtual void            CreateTimeSerie (){;}
    virtual void            Debug ();
    void                    ForceXStep (double step) {aXDefStep = step;}
    void                    Paint (MHDC* dc);
    virtual bool            QueryLeft (int x, int y, MHPoint* point);
    virtual bool            QueryRight (int x, int y, MHPoint* point);
    void                    Recalc ();
    void                    Resize (int x, int y, int w, int h, MHDC *pDC);

    virtual void            DrawChart (int nChart, int color) = 0;
    virtual void            DrawXLabels() = 0;
    virtual int             GetMinMax () = 0;

    int                     GetCanvasWidth () {return (int)aCanvasWidth;}
    virtual void            GetDateRange (MHString* startDate, MHString* stopDate){;}
    int                     GetRange () {return aRangeSize;}
    MHChartSettings*        GetSettings () {return &aSettings;}
    int                     GetSize () {return aRangeMax;}

    virtual void            SetDateRange(const char* pStartDate, const char* pStopDate){;}
    void                    SetStart (int nIndex) {aRangeStart = nIndex;}

protected:
    bool                    aInit;
    double                  aBorder;
    double                  aCanvasHeight;
    double                  aCanvasWidth;
    double                  aXDefStep;
    double                  aXMax;
    double                  aXMin;
    double                  aXPixelRatio;
    double                  aXSpan;
    double                  aXStart;
    double                  aXStep;
    double                  aYLabelWidth;
    double                  aYMax[2];
    double                  aYMin[2];
    double                  aYPixelRatio[2];
    double                  aYStep[2];
    int                     aX;
    int                     aY;
    int                     aW;
    int                     aH;
    int                     aCharHeight;
    int                     aCharWidth;
    int                     aCurrentY;
    int                     aID;
    int                     aOrgSize;
    int                     aRangeMax;
    int                     aRangeSize;
    int                     aRangeStart;
    int                     aSerieCount;
    int                     aXOffset;
    int                     aValid[2];
    MHDC*                   aDC;
    MHChartSettings         aSettings;
    MHString                aLabels[MAX_SERIES];
    MHVector*               aPoints[MAX_SERIES];
    int                     aPositions[MAX_SERIES];
    int                     aTypes[MAX_SERIES];

    virtual bool            AddData (const char* name, int type, MHVector* ve, int align);
    void                    DrawLabel (int nIndex, int nColor, int nRow, int labelLen);
    double                  DrawLine (double X1, double Y1, double X2, double Y2);
    double                  DrawRect (double X1, double Y1, double X2, double Y2, bool flag);
    void                    DrawText (const char* s, double X, double Y);
    void                    DrawYLabels (bool bDrawBox);
    void                    SetXTick ();
    double                  TranslateX (double nX);
    double                  TranslateX2World (double nX);
    double                  TranslateY (double nY);
    double                  TranslateY2World (double nY);

private:
    void                    testRange ();
    void                    defineAxis (int align);
};

#endif
