/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#ifndef MHDouble_h
#define MHDouble_h

#include <MH.h>
#include <MHString.h>



/**
*  MHDouble is a class to formate numbers to strings in a nice formated way.
*  <PRE>
*  Example:
*  MHDouble   d;
*  string     s;
*
*  d.SetSize (0, 0.5);
*  d.ToString (0.77, s)
*  //  String is now 0.70
*  </PRE>
*/
class MHDouble : public MH {
public:
                    MHDouble (){;}
                    MHDouble (double d1, double d2);

    const char*     Class () {return "MHDouble";}
    int             GetIntSize ();
    int             GetFracSize ();
    int             GetSize ();
    double          Round (double d);
    void            SetSize (double d);
    void            SetSize (double d1, double d2);
    void            SetSize (int d1, int d2);
    void            ToString (double number, MHString* s);

private:
    double          itsDouble;
    int             itsIntegerSize;
    int             itsFractionSize;
    int             itsNegFlag;

    void            setFractionSize (double d);
    void            setIntegerSize (double d);
};



/**
*  Create object and set values.
*  @param double - Integer size
*  @param double - Fraction size
*/
inline MHDouble::MHDouble (double d1, double d2) {
    SetSize (d1, d2);
}



/**
*  Return integer number width
*  @return int - The integer width
*/
inline int MHDouble::GetIntSize () {
    return itsIntegerSize;
}



/**
*  Return fractional number width
*  @return int - The fractional size width
*/
inline int MHDouble::GetFracSize () {
    return itsFractionSize;
}



/**
*  Return number width. It's integer width and fraction width.
*  @return int - Total width
*/
inline int MHDouble::GetSize() {
    return itsIntegerSize + itsFractionSize;
}



/**
*  Set width of number
*  @param double - Number to use as a base
*/
inline void MHDouble::SetSize (double d) {
    setFractionSize(d);
    setIntegerSize(d);
}



/**
*  Set width of number
*  @param double - Integer Size
*  @param double - Fraction Size
*/
inline void MHDouble::SetSize (double d1, double d2) {
    setFractionSize(d2);
    setIntegerSize(d1);
}



/**
*  Set width of number
*  @param int - Integer Size
*  @param int - Fraction Size
*/
inline void MHDouble::SetSize (int d1, int d2) {
    itsFractionSize = d2;
    itsIntegerSize = d1;
}



/**
*  Convert number to string with fixed integer size and decimals
*  @param double  - Number to convert
*  @param string& - Destination string
*/
inline void MHDouble::ToString (double number, MHString* s) {
    char Buff[51];

    SNPRINTF (Buff, 50, "%.*f", itsFractionSize, number);
    *s = Buff;
}

#endif
