/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#ifndef MHUtil_h
#define MHUtil_h

#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <math.h>
#include <ctype.h>

#ifdef WIN32
    #pragma warning(disable: 4786)

    #ifdef DM
        #include <fstream.h>
        #include <sys/timeb.h>
        #include <dos.h>
        #include <sys/stat.h>
        #include <io.h>

        #define SLEEP(a)            sleep(a * 100);
    #else
        #include <fstream>
        #include <sys/timeb.h>
        #include <dos.h>
        #include <sys/stat.h>
        #include <io.h>

        #define SLEEP(a)            _sleep(a * 100);
    #endif

    #define STRCASECMP(a,b)     _stricmp (a, b)
    #define STRNCASECMP(a,b,c)  _strnicmp (a, b, c)
    #define STRNCMP(a,b,c)      strncmp (a, b, c)
    #define STRSTR              strstr
    #define STRLWR              _strlwr
    #define SNPRINTF            _snprintf
    #define TOASCII(a)          __toascii ((int)a)
    #define VSNPRINTF           _vsnprintf
    #define __VERSION__         "MSVC 6.0"
#else
    //#include <iostream>
    //#include <ostream>
    //#include <fstream>
    #include <unistd.h>
    #include <sys/time.h>
    #include <sys/types.h>
    #include <sys/stat.h>
    #include <fcntl.h>

    #define STRCASECMP(a,b)     strcasecmp (a, b)
    #define STRNCASECMP(a,b,c)  strncmp (a, b, c)
    #define STRNCAT(a,b,c)      strncat (a, b, c)
    #define STRNCMP(a,b,c)      strncmp (a, b, c)
    #define STRSTR              strstr
    #define STRLWR              strlwr
    #define SNPRINTF            snprintf
    #define TOASCII(a)          toascii ((int)a)
    #define VSNPRINTF           vsnprintf
    #define SLEEP(a)            usleep(int(a * 1000000));
#endif

#define ALLOCATE(a,b)       char* a = new char[b]; *a = '\0';
#define STRNCPY(a,b,c)      {strncpy (a, b, c);a[c]='\0';}
#define STRLEN(sss)         (sss && *sss) ? strlen(sss) : 0
#define CHECK_STRING(sss)   (sss && *sss) ? sss : ""



/**
*  Utility methods. All methods are static.
*/
class MHUtil {
public:
    enum ALIGN {
                            LEFT = 1,
                            RIGHT,
                            CENTER,
    };

    static char**           Allocate (int rows, int size);
    static char*            Allocate (char* old, int& size);
    static char*            AppendString (char* s1, const char* s2, int& len1);
    static char*            AppendChar (char* s1, char s2, int& len1);
    static double           BondValue (double interest_value, double interest, double maturity, int years_maturity);
    static double           Change (double a, double b);
    static double           ChangeVal (double a, double b);
    static int              CountRowsInFile (const char* fileName);
    static void             Delete (char** s, int rows);
    static bool             Equal1 (double d1, double d2);
    static bool             Equal3 (double d1, double d2);
    static bool             Equal5 (double d1, double d2);
    static bool             FileExist (const char* fileName);
    static int              FileSize (const char* fileName);
    static double           Interest (const char* d1, const char* d2, double nInterest);
    static double           Interest (const char* d1, const char* d2, double nValue, double nInterest);
    static double           PercentOf (double a, double b);
    static char*            ReadFile (const char* fileName, int& size);
    static void             WriteFile (const char* fileName, const char* buffer, int size, bool compress);

private:
    static int              aaInitCompress;
};




/**
*  Return change of value
*  @param double  - value
*  @param double  - percent
*/
inline double MHUtil::ChangeVal (double val, double per) {
    return ((per / 100) + 1) * val;

}



/**
*  Compare two doubles with 1 decimal
*  @param double - First double number
*  @param double - Second double number
*  @return bool  - true if number is inside 0.1 range
*/
inline bool MHUtil::Equal1 (double n1, double n2) {
    return (fabs(n1 - n2) < 0.1) ? true : false;
}



/**
*  Compare two doubles with 3 decimals
*  @param double - First double number
*  @param double - Second double number
*  @return bool  - true if number is inside 0.001 range
*/
inline bool MHUtil::Equal3 (double n1, double n2) {
    return (fabs(n1 - n2) < 0.001) ? true : false;
}



/**
*  Compare two doubles with 5 decimals
*  @param double - First double number
*  @param double - Second double number
*  @return bool  - true if number is inside 0.00001 range
*/
inline bool MHUtil::Equal5 (double n1, double n2) {
    return (fabs(n1 - n2) < 0.00001) ? true : false;
}

#endif

