/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#ifndef Fl_Layout_h
#define Fl_Layout_h

#include <FL/Fl_Group.h>




/**
*  A border layout widget or a split widget for two child widgets or
*  a vertical/horizontal group widget..
*/
class Fl_Layout : public Fl_Group {
public:
    enum LAYOUT {
                            BORDER = 1,
                            HORIZONTAL,
                            HORIZONTAL_SPLIT,
                            VERTICAL,
                            VERTICAL_SPLIT,
    };

    enum POSITION {
                            NORTH = 0,
                            SOUTH,
                            EAST,
                            WEST,
                            CENTER,
    };

                            Fl_Layout (LAYOUT layout, int x, int y, int w, int h);

    void                    add (Fl_Widget* widget);
    void                    add (POSITION position, Fl_Widget* widget);
    int                     handle (int event);
    void                    resize (int x, int y, int w, int h);
    void                    space (int space, int north, int south, int west, int east) {aSpace = space; aNorth = north; aSouth = south; aWest = west; aEast = east;}

private:
    int                     aLayout;
    int                     aSize1;
    int                     aSize2;
    int                     aDrag;
    int                     aSpace;
    int                     aNorth;
    int                     aSouth;
    int                     aWest;
    int                     aEast;
    Fl_Widget*              aWidget1;
    Fl_Widget*              aWidget2;
    Fl_Widget*              aWidgets[5];

    void                    resizeBorder (int x, int y, int w, int h);
    void                    resizeHorizontal (int x, int y, int w, int h);
    void                    resizeHorizontalSplit (int x, int y, int w, int h);
    void                    resizeVertical (int x, int y, int w, int h);
    void                    resizeVerticalSplit (int x, int y, int w, int h);
};

#endif
