/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#ifndef BaseChart_h
#define BaseChart_h

#include <report/ReportTypes.h>
#include <Enums.h>
#include <MHDate.h>
#include <MHString.h>

class MHDateChart;
class MHDC;



/**
*  Base class for reports. All child classes must implement Create.
*  Creater of the report must delete the BaseChartFormat object when finished.
*/
class BaseChart {
public:

                            BaseChart (MHDC* dc, int view, MHDate::DAYS timeType);
    virtual                 ~BaseChart () {delete aData;}

    virtual const char*     Class () {return "BaseChart";}
    virtual XTRADER_DATE    ChangeDate () {return XTRADER_BOTH_DATE;}
    void                    CheckDate ();
    virtual ReportTypes*    Choice () {return &aChoice;}
    virtual void            Create () = 0;
    MHDC*                   GetChart () {return aDC;}
    MH*                     GetData () {return aData;}
    MHString*               GetStartDate () {return &aStartDate;}
    MHString*               GetStopDate () {return &aStopDate;}
    virtual void            Reload (const char* message){;}
    void                    SetData (MH* data) {delete aData; aData = data;}
    void                    SetDC (MHDC* dc) {aDC = dc;}
    void                    SetStartDate (const char* startDate) {aStartDate = startDate;}
    void                    SetStopDate (const char* stopDate) {aStopDate = stopDate;}

protected:
    XTRADER_CHART_WINDOW    aWindows[XTRADER_LAST_WINDOW + 1];
    XTRADER_CHART_LAYOUT    aLayout;
    int                     aView;
    int                     aLineWidth;
    int                     aBGColor;
    MHDate::DAYS            aTimeType;
    bool                    aCurrAlign;
    ReportTypes             aChoice;
    MHDC*                   aDC;
    MH*                     aData;
    MHString                aStartDate;
    MHString                aStopDate;
    MHDateChart*            aCurrChart;

    void                    copy (MHVector* v1, MHVector* v2, MHVector* ve1, MHVector* ve2);
    MHDateChart*            createChart (bool label);
    void                    createView ();
    void                    getChart (XTRADER_CHART_WINDOW win);
};

#endif
