/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#include "chart/PortfolioChart.h"
#include "db/Account.h"
#include "db/DBFactory.h"
#include "db/Portfolio.h"
#include "db/Portfolio2.h"
#include "db/Price.h"
#include "db/Security.h"
#include "report/PortfolioValue.h"
#include "Resource.h"
#include <MHDate.h>
#include <MHDateChart.h>
#include <MHDC.h>
#include <MHDebug.h>
#include <MHPoint.h>
#include <Progress.h>



/**
*
*/
PortfolioChart::PortfolioChart (MHDC* dc, int view) :
BaseChart (dc, view, MHDate::FRIDAY) {
    MHDate start;

    start.Set (start.GetYear(), 1, 1);
    aStartDate = start.Get (MHDate::LONG_DATE);

    aChoice.aNames.Push (new MHString (STRING(NO_CHART)));
    aChoice.aTypes.Push (new MHString ((int)NO_CHART));

    aChoice.aNames.Push (new MHString (STRING(PORTFOLIO_PERFORMANCE)));
    aChoice.aTypes.Push (new MHString ((int)PERFORMANCE));

    aChoice.aNames.Push (new MHString (STRING(PORTFOLIO_VALUE)));
    aChoice.aTypes.Push (new MHString ((int)VALUE));

    aChoice.aNames.Push (new MHString (STRING(RELATIVE_)));
    aChoice.aTypes.Push (new MHString ((int)RELATIVE_));

    Reload ("PrefChartArea");
}



/**
*  Create new charts
*/
void PortfolioChart::Create () {
    DBG ("Create")
    DBG_LOG2 ("aLayout=%d  aLineWidth=%d", aLayout, aLineWidth);

    if (!aDC || !aData) return;
    createView();

    //  Draw as many chart areas as user has selected
    for (int f = XTRADER_DEFAULT_WINDOW; f <= XTRADER_LAST_WINDOW; f++) {
        getChart ((XTRADER_CHART_WINDOW)f);
        if (!aCurrChart) continue;

        //  Create chart
        int chart = NO_CHART;

        if (aWindows[f] >= 0 && aWindows[f] < aChoice.aTypes.Size())
            chart = ((MHString*)aChoice.aTypes[aWindows[f]])->ToInt();

        switch (chart) {
            case PERFORMANCE:
                createPerformance ();
                break;

            case RELATIVE_:
                createRelative ();
                break;

            case VALUE:
                createValue ();
                break;

            case NO_CHART:
                aDC->Align (aCurrAlign);
                break;

            default:
                break;
        }
    }
}




/**
*  Create a performance chart for portfolio
*/
void PortfolioChart::createPerformance () {
    MHVector*       v1   = 0;
    MHVector*       ve1  = new MHVector ();
    MHVector*       ve2  = new MHVector ();
    MHVector*       ve3  = new MHVector ();
    MHVector*       ve4  = new MHVector ();
    MHVector*       ve5  = new MHVector ();
    MHVector*       ve6  = new MHVector ();
    Account*        acc  = (Account*) aData;
    int             id   = DBFactory::GetPrefIntValue (STRING(PREF_CHARTP_RELATIVE), aView, 0, 999999999, 0);
    Security        stmp(id, "", "");
    Security*       sec  = (Security*) DBFactory::LoadSecurity()->Find (&stmp, Security::SORT_ID);
    char            buff[100];
    PortfolioValue  value;

    Portfolio2::LoadPortfolio (acc, ve1);
    Portfolio2::Performance (ve1, ve2, ve3, 0, &aStartDate, &aStopDate, &value);
    copy (ve3, 0, ve4, 0);

    if (sec && ve4->Size() > 0) {
        MHPoint* p1 = (MHPoint*) ve4->At (0);
        v1 = DBFactory::LoadPrice (sec);
        copy (v1, 0, ve5, 0);
        MHPoint::Normalise (ve5, ve6, p1->GetHigh());
        if (ve6->Size() > ve4->Size())
            MHPoint::Cut (ve4, ve6);
    }

    #ifdef BUGG
        MHPoint::Save ("dbg_portfolio_chart.txt", ve4);
    #endif

    SNPRINTF (buff, 99, "%s %s", acc->GetAccount()->Get(), STRING(PORTFOLIO_PERFORMANCE));
    aCurrChart->AddData (buff, MHDateChart::LINE_CHART, ve4, aCurrAlign);
    if (sec)
        aCurrChart->AddData (sec->GetName()->Get(), MHDateChart::LINE_CHART, ve6, aCurrAlign);
    else
        delete ve6;
    ve1->Erase ();
    ve2->Erase ();
    ve3->Erase ();
    ve5->Erase ();
    delete ve1;
    delete ve2;
    delete ve3;
    delete ve5;
}



/**
*  Create a relative price chart
*/
void PortfolioChart::createRelative () {
    MHVector*       v1   = 0;
    MHVector*       ve1  = new MHVector ();
    MHVector*       ve2  = new MHVector ();
    int             id   = DBFactory::GetPrefIntValue (STRING(PREF_CHARTP_RELATIVE), aView, 0, 999999999, 0);
    Security        stmp(id, "", "");
    Security*       sec  = (Security*) DBFactory::LoadSecurity()->Find (&stmp, Security::SORT_ID);

    if (sec) v1 = DBFactory::LoadPrice (sec);
    copy (v1, 0, ve1, 0);
    MHPoint::Normalise (ve1, ve2, 100);

    if (sec)
        aCurrChart->AddData (sec->GetName()->Get(), MHDateChart::LINE_CHART, ve2, aCurrAlign);
    else
        delete ve2;
    ve1->Erase ();
    delete ve1;
}



/**
*  Create a value chart for portfolio
*/
void PortfolioChart::createValue () {
    MHVector*       ve1  = new MHVector ();
    MHVector*       ve2  = new MHVector ();
    MHVector*       ve3  = new MHVector ();
    MHVector*       ve4  = new MHVector ();
    Account*        acc  = (Account*) aData;
    PortfolioValue  value;
    char            buff[100];

    Portfolio2::LoadPortfolio (acc, ve1);
    Portfolio2::Performance (ve1, ve2, 0, ve3, &aStartDate, &aStopDate, &value);
    copy (ve3, 0, ve4, 0);

    #ifdef BUGG
        MHPoint::Save ("dbg_portfolio_chart.txt", ve4);
    #endif

    SNPRINTF (buff, 99, "%s %s", acc->GetAccount()->Get(), STRING(PORTFOLIO_VALUE));
    aCurrChart->AddData (buff, MHDateChart::LINE_CHART, ve4, aCurrAlign);
    ve1->Erase ();
    ve2->Erase ();
    ve3->Erase ();
    delete ve1;
    delete ve2;
    delete ve3;}



/**
*  Check is the message is for me and create chart again if so
*/
void PortfolioChart::Reload (const char* message) {
    DBG ("Reload")

    if (message == 0 || *message == '\0')
        Create();
    else if (strcmp (message, "Price") == 0)
        Create();
    else if (strcmp (message, "Portfolio") == 0)
        Create();
    else if (strcmp (message, "PrefChartArea") == 0) {
        //  Load all default settings from preference file
        aLayout = (XTRADER_CHART_LAYOUT) DBFactory::GetPrefIntValue (STRING(PREF_CHARTP_LAYOUT), aView, XTRADER_DEFAULT_LAYOUT, XTRADER_LAST_LAYOUT, XTRADER_DEFAULT_LAYOUT);
        aLineWidth = DBFactory::GetPrefIntValue (STRING(PREF_CHARTP_LINEWIDTH), aView, XTRADER_MIN_LINE_WIDTH, XTRADER_MAX_LINE_WIDTH, XTRADER_DEFAULTP_LINE_WIDTH);
        aBGColor = DBFactory::GetPrefIntValue (STRING(PREF_BGPCOLOR), aView, 0, 255, 0);

        for (int f = XTRADER_DEFAULT_WINDOW; f <= XTRADER_LAST_WINDOW; f++) {
            char b[100];
            SNPRINTF (b, 99, "%s_%d_%d", STRING(PREF_CHARTP_WINDOW), aView, f);
            aWindows[f] = (XTRADER_CHART_WINDOW) DBFactory::GetPrefIntValue (b, PortfolioChart::NO_CHART, PortfolioChart::LAST_CHART, PortfolioChart::NO_CHART);
        }

        DBG_LOG3 ("aLayout=%d  aLineWidth=%d  aTimeType=%d", aLayout, aLineWidth, aTimeType);

        Create();
    }
}
