/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#include "chart/PriceChart.h"
#include "db/DBFactory.h"
#include "db/Price.h"
#include "db/Security.h"
#include "Resource.h"
#include "Progress.h"
#include <MHPoint.h>
#include <MHDate.h>
#include <MHDateChart.h>
#include <MHDC.h>
#include <MHDebug.h>



/**
*
*/
PriceChart::PriceChart (MHDC* dc, int view, MHDate::DAYS timeType) :
BaseChart (dc, view, timeType) {
    aChoice.aNames.Push (new MHString (STRING(NO_CHART)));
    aChoice.aTypes.Push (new MHString ((int)NO_CHART));

    aChoice.aNames.Push (new MHString (STRING(BAR)));
    aChoice.aTypes.Push (new MHString ((int)BAR));

    aChoice.aNames.Push (new MHString (STRING(LINE)));
    aChoice.aTypes.Push (new MHString ((int)LINE));

    aChoice.aNames.Push (new MHString (STRING(MOVING_AVERAGE)));
    aChoice.aTypes.Push (new MHString ((int)MOVING_AVERAGE));

    aChoice.aNames.Push (new MHString (STRING(ATR)));
    aChoice.aTypes.Push (new MHString ((int)ATR));

    aChoice.aNames.Push (new MHString (STRING(BOLLINGER)));
    aChoice.aTypes.Push (new MHString ((int)BOLLINGER));

    aChoice.aNames.Push (new MHString (STRING(MOMENTUM)));
    aChoice.aTypes.Push (new MHString ((int)MOMENTUM));

    aChoice.aNames.Push (new MHString (STRING(RELATIVE_)));
    aChoice.aTypes.Push (new MHString ((int)RELATIVE_));

    aChoice.aNames.Push (new MHString (STRING(RSI)));
    aChoice.aTypes.Push (new MHString ((int)RSI));

    aChoice.aNames.Push (new MHString (STRING(STDDEV)));
    aChoice.aTypes.Push (new MHString ((int)STDDEV));

    aChoice.aNames.Push (new MHString (STRING(STOCHASTICS)));
    aChoice.aTypes.Push (new MHString ((int)STOCHASTICS));

    aChoice.aNames.Push (new MHString (STRING(VOLUME)));
    aChoice.aTypes.Push (new MHString ((int)VOLUME));

    Reload ("PrefChartArea");
}



/**
*  Create new charts
*/
void PriceChart::Create () {
    DBG ("Create")
    DBG_LOG2 ("aLayout=%d  aLineWidth=%d", aLayout, aLineWidth);

    if (!aDC || !aData) return;
    createView();

    //  Draw as many chart areas as user has selected
    for (int f = XTRADER_DEFAULT_WINDOW; f <= XTRADER_LAST_WINDOW; f++) {
        getChart ((XTRADER_CHART_WINDOW)f);
        if (!aCurrChart) continue;

        //  Create chart
        int chart = NO_CHART;

        if (aWindows[f] >= 0 && aWindows[f] < aChoice.aTypes.Size())
            chart = ((MHString*)aChoice.aTypes[aWindows[f]])->ToInt();

        switch (chart) {
            case ATR:
                createAtr ();
                break;

            case BAR:
                createBar ();
                break;

            case BOLLINGER:
                createBollinger ();
                break;

            case LINE:
                createLine ();
                break;

            case MOMENTUM:
                createMomentum ();
                break;

            case MOVING_AVERAGE:
                createMovingAverage ();
                break;

            case RELATIVE_:
                createRelative ();
                break;

            case RSI:
                createRSI ();
                break;

            case STOCHASTICS:
                createStochastics ();
                break;

            case STDDEV:
                createStdDev ();
                break;

            case VOLUME:
                createVolume ();
                break;

            case NO_CHART:
                aDC->Align (aCurrAlign);
                break;

            default:
                break;
        }
    }
}




/**
*  Create a atr chart for a security.
*/
void PriceChart::createAtr () {
    int          mvg = DBFactory::GetPrefIntValue (STRING(PREF_MVG_ATR), aView, 3, 100, MHPoint::MVG_ATR);
    char         buff[100];
    MHVector*    v1   = 0;
    MHVector*    v2   = 0;
    MHVector*    ve1  = new MHVector ();
    MHVector*    ve2  = new MHVector ();
    MHVector*    ve3  = new MHVector ();
    Security*    sec  = (Security*) aData;

    v1 = DBFactory::LoadPrice (sec);
    v2 = DBFactory::LoadClosedPrice (sec);
    copy (v1, v2, ve1, ve2);

    MHPoint::ATR (ve1, ve3, mvg);

    sprintf (buff, "%s %s %d %s", sec->GetName()->Get(), STRING(ATR), mvg, STRING(DAYS));
    aCurrChart->AddData (buff, MHDateChart::LINE_CHART, ve3, aCurrAlign);
    aCurrChart->SetBlockList (ve2);

    ve1->Erase ();
    delete ve1;
}



/**
*  Create a bar chart
*/
void PriceChart::createBar () {
    MHVector*    v1   = 0;
    MHVector*    v2   = 0;
    MHVector*    ve1  = new MHVector ();
    MHVector*    ve2  = new MHVector ();
    Security*    sec  = (Security*) aData;

    v1 = DBFactory::LoadPrice (sec);
    v2 = DBFactory::LoadClosedPrice (sec);
    copy (v1, v2, ve1, ve2);

    aCurrChart->AddData (sec->GetName()->Get(), MHDateChart::BAR_CHART, ve1, aCurrAlign);
    aCurrChart->SetBlockList (ve2);
}



/**
*  Create a bollinger chart for a security.
*/
void PriceChart::createBollinger () {
    char         buff[100];
    int          mvg  = DBFactory::GetPrefIntValue (STRING(PREF_MVG_BOLLINGER), aView, 3, 100, MHPoint::MVG_BOLLINGER);
    MHVector*    v1   = 0;
    MHVector*    v2   = 0;
    MHVector*    ve1  = new MHVector ();
    MHVector*    ve2  = new MHVector ();
    MHVector*    ve3  = new MHVector ();
    MHVector*    ve4  = new MHVector ();
    MHVector*    ve5  = new MHVector ();
    Security*    sec  = (Security*) aData;

    v1 = DBFactory::LoadPrice (sec);
    v2 = DBFactory::LoadClosedPrice (sec);
    copy (v1, v2, ve1, ve2);
    MHPoint::Bollinger (v1, ve3, ve4, ve5, mvg);

    SNPRINTF (buff, 99, "%s %s %d %s", sec->GetName()->Get(), STRING(BOLLINGER), mvg, STRING(DAYS));
    aCurrChart->AddData (buff, MHDateChart::LINE_CHART, ve3, aCurrAlign);
    aCurrChart->AddData (STRING(UPPER_BAND), MHDateChart::LINE_CHART, ve4, aCurrAlign);
    aCurrChart->AddData (STRING(LOWER_BAND), MHDateChart::LINE_CHART, ve5, aCurrAlign);
    aCurrChart->SetBlockList (ve2);

    ve1->Erase ();
    delete ve1;
}



/**
*  Create a line chart
*/
void PriceChart::createLine () {
    MHVector*    v1   = 0;
    MHVector*    v2   = 0;
    MHVector*    ve1  = new MHVector ();
    MHVector*    ve2  = new MHVector ();
    Security*    sec  = (Security*) aData;

    v1 = DBFactory::LoadPrice (sec);
    v2 = DBFactory::LoadClosedPrice (sec);
    copy (v1, v2, ve1, ve2);

    aCurrChart->AddData (sec->GetName()->Get(), MHDateChart::LINE_CHART, ve1, aCurrAlign);
    aCurrChart->SetBlockList (ve2);
}



/**
*  Create a momentum chart for a security.
*/
void PriceChart::createMomentum () {
    int          mvg = DBFactory::GetPrefIntValue (STRING(PREF_MVG_MOMENTUM), aView, 3, 100, MHPoint::MVG_MOMENTUM);
    char         buff[100];
    MHVector*    v1   = 0;
    MHVector*    v2   = 0;
    MHVector*    ve1  = new MHVector ();
    MHVector*    ve2  = new MHVector ();
    MHVector*    ve3  = new MHVector ();
    MHVector*    ve4  = new MHVector ();
    Security*    sec  = (Security*) aData;

    v1 = DBFactory::LoadPrice (sec);
    v2 = DBFactory::LoadClosedPrice (sec);
    copy (v1, v2, ve1, ve2);
    MHPoint::Momentum (ve1, ve3, ve4, mvg);

    SNPRINTF (buff, 99, "%s %s %d %s", sec->GetName()->Get(), STRING(MOMENTUM), mvg, STRING(DAYS));
    aCurrChart->AddData (buff, MHDateChart::LINE_CHART, ve3, aCurrAlign);
    aCurrChart->AddData ("", MHDateChart::LINE_CHART, ve4, aCurrAlign);
    aCurrChart->SetBlockList (ve2);

    ve1->Erase ();
    delete ve1;
}



/**
*  Create a bar chart with two moving averages.
*/
void PriceChart::createMovingAverage () {
    char         buff[100];
    int          mvg1 = DBFactory::GetPrefIntValue (STRING(PREF_MVG_SHORT), aView, 3, 100, MHPoint::MVG_SHORT);
    int          mvg2 = DBFactory::GetPrefIntValue (STRING(PREF_MVG_LONG), aView, 3, 100, MHPoint::MVG_LONG);
    MHVector*    v1   = 0;
    MHVector*    v2   = 0;
    MHVector*    ve1  = new MHVector ();
    MHVector*    ve2  = new MHVector ();
    MHVector*    ve3  = new MHVector ();
    MHVector*    ve4  = new MHVector ();
    Security*    sec  = (Security*) aData;

    v1 = DBFactory::LoadPrice (sec);
    v2 = DBFactory::LoadClosedPrice (sec);
    copy (v1, v2, ve1, ve2);

    MHPoint::MovingAverage (ve1, ve3, mvg1);
    MHPoint::MovingAverage (ve1, ve4, mvg2);

    aCurrChart->AddData (sec->GetName()->Get(), MHDateChart::BAR_CHART, ve1, aCurrAlign);
    aCurrChart->SetBlockList (ve2);

    SNPRINTF (buff, 99, "%s %d %s", STRING(MVG), mvg1, STRING(DAYS));
    aCurrChart->AddData (buff, MHDateChart::LINE_CHART, ve3, aCurrAlign);

    SNPRINTF (buff, 99, "%s %d %s", STRING(MVG), mvg2, STRING(DAYS));
    aCurrChart->AddData (buff, MHDateChart::LINE_CHART, ve4, aCurrAlign);

}



/**
*  Create a chart where you compare two securities with each other and convert price data to start at 100
*/
void PriceChart::createRelative () {
    int          id   = DBFactory::GetPrefIntValue (STRING(PREF_CHART_RELATIVE), aView, 0, 999999999, 0);
    MHVector*    v1   = 0;
    MHVector*    v2   = 0;
    MHVector*    v3   = 0;
    MHVector*    v4   = 0;
    MHVector*    ve1  = new MHVector ();
    MHVector*    ve2  = new MHVector ();
    MHVector*    ve3  = new MHVector ();
    MHVector*    ve4  = new MHVector ();
    MHVector*    ve5  = new MHVector ();
    MHVector*    ve6  = new MHVector ();
    Security*    sec  = (Security*) aData;
    Security     stmp(id, "", "");
    Security*    sec2 = (Security*) DBFactory::LoadSecurity()->Find (&stmp, Security::SORT_ID);

    v1 = DBFactory::LoadPrice (sec);
    v2 = DBFactory::LoadClosedPrice (sec);
    copy (v1, v2, ve1, ve2);

    if (sec2) {
        v3 = DBFactory::LoadPrice (sec2);
        v4 = DBFactory::LoadClosedPrice (sec2);
        copy (v3, v4, ve3, ve4);
        if (ve1->Size() > ve3->Size())
            MHPoint::Cut (ve3, ve1);
        else
            MHPoint::Cut (ve1, ve3);
        MHPoint::Normalise (ve1, ve5, 100);
        MHPoint::Normalise (ve3, ve6, 100);
    }
    else
        MHPoint::Normalise (ve1, ve5, 100);

    aCurrChart->AddData (sec->GetName()->Get(),  MHDateChart::LINE_CHART, ve5, aCurrAlign);
    if (sec2)
        aCurrChart->AddData (sec2->GetName()->Get(), MHDateChart::LINE_CHART, ve6, aCurrAlign);
    else
        delete ve6;
    aCurrChart->SetBlockList (ve4);

    ve1->Erase ();
    delete ve1;
    ve2->Erase ();
    delete ve2;
    ve3->Erase ();
    delete ve3;
}



/**
*  Create a chart using the rsi formula
*/
void PriceChart::createRSI () {
    char         buff[100];
    int          mvg  = DBFactory::GetPrefIntValue (STRING(PREF_MVG_RSI), aView, 3, 100, MHPoint::MVG_RSI);
    MHVector*    v1   = 0;
    MHVector*    v2   = 0;
    MHVector*    ve1  = new MHVector ();
    MHVector*    ve2  = new MHVector ();
    MHVector*    ve3  = new MHVector ();
    Security*    sec  = (Security*) aData;

    v1 = DBFactory::LoadPrice (sec);
    v2 = DBFactory::LoadClosedPrice (sec);
    copy (v1, v2, ve1, ve2);
    MHPoint::RSI (ve1, ve3, mvg);

    SNPRINTF (buff, 99, "%s %s %d %s", sec->GetName()->Get(), STRING(RSI), mvg, STRING(DAYS));
    aCurrChart->AddData (buff, MHDateChart::LINE_CHART, ve3, aCurrAlign);
    aCurrChart->SetBlockList (ve2);

    ve1->Erase ();
    delete ve1;
}



/**
*  Create a standard deviation chart for a security.
*/
void PriceChart::createStdDev () {
    char         buff[100];
    int          mvg  = DBFactory::GetPrefIntValue (STRING(PREF_MVG_STDDEV), aView, 3, 100, MHPoint::MVG_STD);
    MHVector*    v1   = 0;
    MHVector*    v2   = 0;
    MHVector*    ve1  = new MHVector ();
    MHVector*    ve2  = new MHVector ();
    MHVector*    ve3  = new MHVector ();
    Security*    sec  = (Security*) aData;

    v1 = DBFactory::LoadPrice (sec);
    v2 = DBFactory::LoadClosedPrice (sec);
    copy (v1, v2, ve1, ve2);
    MHPoint::StdDev (ve1, ve3, mvg);

    SNPRINTF (buff, 99, "%s %s %d %s", sec->GetName()->Get(), STRING(STDDEV), mvg, STRING(DAYS));
    aCurrChart->AddData (buff, MHDateChart::LINE_CHART, ve3, aCurrAlign);
    aCurrChart->SetBlockList (ve2);

    ve1->Erase ();
    delete ve1;
}



/**
*  Create a chart using the stochastics formula.
*/
void PriceChart::createStochastics () {
    char         buff[100];
    int          mvgk = DBFactory::GetPrefIntValue (STRING(PREF_MVG_STOCHASTICK), aView, 3, 100, MHPoint::MVG_STOCHASTICS_K);
    int          mvgd = DBFactory::GetPrefIntValue (STRING(PREF_MVG_STOCHASTICD), aView, 3, 100, MHPoint::MVG_STOCHASTICS_D);
    MHVector*    v1   = 0;
    MHVector*    v2   = 0;
    MHVector*    ve1  = new MHVector ();
    MHVector*    ve2  = new MHVector ();
    MHVector*    ve3  = new MHVector ();
    MHVector*    ve4  = new MHVector ();
    Security*    sec  = (Security*) aData;

    v1 = DBFactory::LoadPrice (sec);
    v2 = DBFactory::LoadClosedPrice (sec);
    copy (v1, v2, ve1, ve2);
    MHPoint::Stochastics (ve1, ve3, mvgk);
    MHPoint::MovingAverage (ve3, ve4, mvgd);

    SNPRINTF (buff, 99, "%s %s %d %s", sec->GetName()->Get(), STRING(STOCHASTICS), mvgk, STRING(DAYS));
    aCurrChart->AddData (buff, MHDateChart::LINE_CHART, ve3, aCurrAlign);
    SNPRINTF (buff, 99, "%s %d %s", STRING(MVG), mvgd, STRING(DAYS));
    aCurrChart->AddData (buff, MHDateChart::LINE_CHART, ve4, aCurrAlign);
    aCurrChart->SetBlockList (ve2);

    ve1->Erase ();
    delete ve1;
}



/**
*  Create a chart with only volume data
*/
void PriceChart::createVolume () {
    char         buff[100];
    MHVector*    v1   = 0;
    MHVector*    v2   = 0;
    MHVector*    ve1  = new MHVector ();
    MHVector*    ve2  = new MHVector ();
    MHVector*    ve3  = new MHVector ();
    Security*    sec  = (Security*) aData;

    v1 = DBFactory::LoadPrice (sec);
    v2 = DBFactory::LoadClosedPrice (sec);
    copy (v1, v2, ve1, ve2);
    MHPoint::Volume (ve1, ve3);

    SNPRINTF (buff, 99, "%s %s", sec->GetName()->Get(), STRING(VOLUME));
    aCurrChart->AddData (buff, MHDateChart::RECT_CHART, ve3, aCurrAlign);
    aCurrChart->SetBlockList (ve2);

    ve1->Erase ();
    delete ve1;
}



/**
*  Check is the message is for me and create chart again if so
*/
void PriceChart::Reload (const char* message) {
    DBG ("Reload")

    if (message == 0 || *message == '\0')
        Create();
    else if (strcmp (message, "Price") == 0)
        Create();
    else if (strcmp (message, "PrefChartArea") == 0) {
        //  Load all default settings from preference file
        aLayout    = (XTRADER_CHART_LAYOUT) DBFactory::GetPrefIntValue (STRING(PREF_CHART_LAYOUT), aView, XTRADER_DEFAULT_LAYOUT, XTRADER_LAST_LAYOUT, XTRADER_DEFAULT_LAYOUT);
        aLineWidth = DBFactory::GetPrefIntValue (STRING(PREF_CHART_LINEWIDTH), aView, XTRADER_MIN_LINE_WIDTH, XTRADER_MAX_LINE_WIDTH, XTRADER_DEFAULT_LINE_WIDTH);
        aBGColor   = DBFactory::GetPrefIntValue (STRING(PREF_BGCOLOR), aView, 0, 255, 0);

        for (int f = XTRADER_DEFAULT_WINDOW; f <= XTRADER_LAST_WINDOW; f++) {
            char b[50];
            sprintf (b, "%s_%d_%d", STRING(PREF_CHART_WINDOW), aView, f);
            aWindows[f] = (XTRADER_CHART_WINDOW) DBFactory::GetPrefIntValue (b, PriceChart::NO_CHART, PriceChart::LAST_CHART, PriceChart::NO_CHART);
        }

        DBG_LOG3 ("aLayout=%d  aLineWidth=%d  aTimeType=%d", aLayout, aLineWidth, aTimeType);

        Create();
    }
}
