/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#include <download/Controller.h>
#include <Progress.h>



/**
*
*/
::Controller::Controller () {
    aStatus = aDone = aCancel = false;
    aPrices = 0;
}



/**
*
*/
::Controller::~Controller () {
    aItems.Erase();
    if (aPrices) {
        aPrices->Erase();
        delete aPrices;
    }
}



/**
*
*/
void Controller::SetParams (MHVector* sets, BaseDownload* load, BaseDownload::TYPE type, const char* startDate, const char* stopDate, Controller::ACTION action, const char* fileName) {
    aSets       = sets;
    aLoad       = load;
    aType       = type;
    aAction     = action;
    aStartDate  = startDate;
    aStopDate   = stopDate;
    aFileName   = fileName;
}



/**
*  Go through all sets and download price data
*/
bool Controller::Start () {
    int     total   = 0;
    int     rows    = 0;

    aStatus = true;
    aDone   = false;
    aCancel = false;

    DBFactory::WriteLog ("Starting download");

    // Count number of objects, for the progress
    for (int i = 0; i < aSets->Size(); i++) {
        Set*        set = (Set*) (*aSets)[i];
        DBFactory::LoadSetItem (set, &aItems);
        total += aItems.Size();
        aItems.Erase ();
    }
    xtrader_reset (total);

    if (aFileName)
        remove (aFileName);

    for (int f = 0; f < aSets->Size(); f++) {
        Set*        set     = (Set*) (*aSets)[f];
        FILE*       file    = 0;

        try {
            if ((aAction == SAVE || aAction == UPDATE_SAVE) && aFileName)
                file = fopen (aFileName, "a");

            if (file) {
                DBFactory::LoadSetItem (set, &aItems);
                SetItem::LoadSecurities (&aItems);

                for (int e = 0; e < aItems.Size(); e++) {
                    SetItem*    item    = (SetItem*) aItems[e];
                    Security*   sec     = item->GetSecurity ();
                    aPrices             = aLoad->Load (aType, aStartDate, aStopDate, sec);

                    Price::Export (sec, aPrices, file);
                    rows += aPrices->Size();
                    aPrices->Erase ();
                    delete aPrices;
                    aPrices = 0;

                    xtrader_add(1);
                    if (xtrader_abort()) {
                        aStatus = false;
                        aCancel = true;
                        break;
                    }
                }
                fclose (file);
            }
        }
        catch (...) {
        }
        DBFactory::WriteLog ("%d rows downloaded", rows);
    }
    aDone = true;
    return aStatus;
}
