/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#ifndef XTrader_h
#define XTrader_h

#include <FL/Fl.h>
#include <FL/fl_ask.h>
#include <FL/fl_draw.h>
#include <FL/Fl_Box.h>
#include <FL/Fl_Double_Window.h>
#include <gui/BaseView.h>
#include <MHMessage.h>
#include <MHString.h>

class Fl_Layout;
class Fl_Menu_Bar;
struct Fl_Menu_Item;
class Fl_Tabs;



/**
*  XTrader is main window object and manages the menu,
*  toolbar, statusbar and the different tab views
*/
class XTrader : public Fl_Double_Window {
public:
                            XTrader (int w, int h);
                            ~XTrader ();

    void                    BuildToolbar ();
    void                    cbClose ();
    static void             cbClose_ (Fl_Double_Window* w, void* data) {XTrader::GetApp()->cbClose ();}
    void                    cbMenu (int id);
    static void             cbMenu_ (Fl_Menu_Bar* w, void* data) {XTrader::GetApp()->cbMenu ((int)data);}
    bool                    Close ();
    static void             Close (BaseView* view) {XTrader::aaApp->close (view);}
    static void             FixButton (Fl_Button* b);
    static int              FontSize ();
    static XTrader*         GetApp () {return aaApp;}
    int                     handle (int event);
    void                    LoadWorkSpace ();
    void                    resize (int x, int y, int w, int h);
    void                    SavePos ();
    void                    SaveWorkSpace ();
    void                    Show (int argc, char** argv);
    static void             SendMessage (const MH* sender, const char* m1 = 0, const char* m2 = 0, const char* m3 = 0, void* m4 = 0) {XTrader::aaApp->aListeners->Send (sender, m1, m2, m3, m4);}
    static void             SetError (const char* message) {STRNCPY (XTrader::aaApp->aBuffer, message, 199); XTrader::aaApp->aStatus->label (XTrader::aaApp->aBuffer); fl_beep (FL_BEEP_ERROR); XTrader::aaApp->aStatus->redraw(); Fl::flush();}
    static void             SetInfo (const char* message) {STRNCPY (XTrader::aaApp->aBuffer, message, 199); XTrader::aaApp->aStatus->label (XTrader::aaApp->aBuffer); fl_beep (FL_BEEP_MESSAGE); XTrader::aaApp->aStatus->redraw(); Fl::flush();}
    static void             SetMessage (const char* message) {STRNCPY (XTrader::aaApp->aBuffer, message, 199); XTrader::aaApp->aStatus->label (XTrader::aaApp->aBuffer); XTrader::aaApp->aStatus->redraw(); Fl::flush();}
    static void             StartWork (const char* message, int showtime = 0);
    static void             StopWork (const char* message);


protected:
    MHMessage*              aListeners;
    BaseView*               aLastClosedView;
    Fl_Tabs*                aTabs;
    Fl_Menu_Bar*            aMenu;
    Fl_Layout*              aLayout;
    Fl_Box*                 aStatus;
    Fl_Layout*              aTop;
    Fl_Layout*              aToolbar;
    static XTrader*         aaApp;
    char                    aBuffer[200];
    MHString                aLabel;

    void                    close (BaseView* view);
    void                    createMenu ();
    BaseView*               createView (const char* view);
    bool                    findView (const char* label);
    Fl_Menu_Item*           getMenuItem (const char* label);
};

#endif
