/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#include <table/PriceData.h>
#include <db/DBFactory.h>
#include <db/Price.h>
#include <db/Security.h>
#include <gui/XTrader.h>
#include <Resource.h>
#include <MHDate.h>
#include <MHString.h>
#include <FL/Fl.h>



/**
*  @param Security* - The security object to load price data for.
*/
PriceData::PriceData (Security* s) :
BaseData () {
    aVector   = new MHVector (500);
    aSecurity = s;
    aChoice   = MHUtil::Allocate (3, 30);

    STRNCPY (aChoice[0], STRING(PRICE), 20)
    STRNCPY (aChoice[1], STRING(CLOSED), 20)
    STRNCPY (aChoice[2], STRING(SPLIT), 20)

    Price::LoadAll (aSecurity, aVector);
}



/**
*
*/
PriceData::~PriceData () {
    MHUtil::Delete (aChoice, 3);
    aVector->Erase ();
    delete aVector;
    delete aSecurity;
}



/**
*
*/
void PriceData::add () {
    insert (aVector->Size());
}



/**
*
*/
Fl_Align PriceData::align (int row, int col) {
    switch (col) {
        case 0:
        case 1:
            return FL_ALIGN_LEFT;

        default:
            return FL_ALIGN_RIGHT;
    }
}



/**
*
*/
const char** PriceData::choice (int row, int col, int& size, int& selected, char* label) {
    Price* p = (Price*) aVector->At (row);

    size = 3;

    if (p) {
        if (p->GetType () == Price::SPLIT)
            selected = 2;
        else if (p->GetType () == Price::CLOSED)
            selected = 1;
        else
            selected = 0;

    }

    return (char const**) aChoice;
}



/**
*
*/
FL_TABLE_EDITOR PriceData::editor_type (int row, int col, bool force_custom) {
    switch (col) {
        case 0:
            return FL_TEXT_EDITOR;

        case 1:
            return FL_LIST_EDITOR;

        default:
            return FL_FLOAT_EDITOR;
    }
}



/**
*
*/
void PriceData::erase (int row) {
    if (row >= 0 && row < aVector->Size()) {
        aVector->Erase (row);
        aDirty = true;
    }
}



/**
*
*/
void PriceData::insert (int row) {
    MHDate now;
    Price* p1 = new Price (now.Get (MHDate::LONG_DATE), Price::PRICE, -1, -1, -1, -1, -1);
    Price* p2 = (Price*) aVector->Search (p1, 0, Price::SORT_DATE_TYPE);

    if (p2) {
        delete p1;
        XTrader::SetError (STRING(ERROR_DATA_ERROR));
        return;
    }
    aVector->Insert (p1, row);
    aDirty = true;
}



/**
*
*/
void PriceData::reload (const char* message) {
}


/**
*
*/
int PriceData::save () {
    for (int f = 0; f < aVector->Size(); f++) {
        Price* p = (Price*) (*aVector)[f];

        switch (p->GetType()) {
            case Price::PRICE:
                if (p->GetClose() > p->GetHigh () ||
                    p->GetClose() < p->GetLow ())
                    return f;
                
                if (p->GetOpen() > 0.00000001) {
                    if (p->GetOpen() > p->GetHigh () ||
                        p->GetOpen() < p->GetLow ())
                        return f;
                }
                break;

            case Price::SPLIT:
                if (p->GetHigh () < 0.0000001)
                    return f;
                break;

            default:
                p->SetHigh (-1.0);
                p->SetLow (-1.0);
                p->SetClose (-1.0);
                p->SetOpen (-1.0);
                p->SetVolume (-1.0);
                break;
        }
    }

    Price::Save (aSecurity, aVector);
    DBFactory::ClearPrice();
    aDirty = false;
    XTrader::SetInfo (STRING(DATA_SAVED));
    XTrader::SendMessage (0, "Price", "changed", "", aSecurity);
    return -1;
}



/**
*
*/
void PriceData::sort (int col, bool ascending) {
}



/**
*
*/
bool PriceData::value (const char* data, int row, int col) {
    if (row < 0 || row >= aVector->Size()) return true;

    Price* s  = (Price*) (*aVector)[row];
    ASS (s)

    switch (col) {
        case 0: {
            if (strcmp (s->GetDate(), data) == 0) return true;
            Price p(data, s->GetType(), 0);
            if (MHDate::ValidDate (data) == false || aVector->Search (&p, 0, Price::SORT_DATE_TYPE) != 0)
                return false;
            s->SetDate (data);
            break;
        }

        case 1: {
            Price::TYPE type;
            if (strcmp(STRING(PRICE), data) == 0)
                type = Price::PRICE;
            else if (strcmp(STRING(SPLIT), data) == 0)
                type = Price::SPLIT;
            else
                type = Price::CLOSED;
            if (type == s->GetType()) return true;
            Price p(s->GetDate(), type, 0);
            if (aVector->Search (&p, 0, Price::SORT_DATE_TYPE) != 0)
                return false;
            s->SetType (type);
            break;
        }

        case 2:
            s->SetHigh (atof(data));
            break;

        case 3:
            s->SetLow (atof(data));
            break;

        case 4:
            s->SetClose (atof(data));
            break;

        case 5:
            s->SetOpen (atof(data));
            break;

        case 6:
            s->SetVolume (atof(data));
            break;
    }

    s->IsChanged (true);
    aDirty = true;
    return true;
}



/**
*
*/
const char* PriceData::value (int row, int col) {
    if (row == FL_LABEL_ROW)
        return aSecurity ? aSecurity->GetName()->Get() : "PriceData::value !!!";
    else if (row == FL_HEADER_ROW) {
        switch (col) {
            case 0:
                return STRING(DATE);

            case 1:
                return STRING(TYPE);

            case 2:
                return STRING(HIGH);

            case 3:
                return STRING(LOW);

            case 4:
                return STRING(CLOSE_);

            case 5:
                return STRING(OPEN);

            case 6:
                return STRING(VOLUME);

            default:
                return "";
        }
    }
    else {
        Price* s = (Price*) aVector->At (row);
        if (!s) return "";

        switch (col) {
            case 0:
                return s->GetDate();

            case 1:
                if (s->GetType() == Price::CLOSED)
                    STRNCPY (aBuff, STRING(CLOSED), 20)
                else if (s->GetType() == Price::SPLIT)
                    STRNCPY (aBuff, STRING(SPLIT), 20)
                else
                    STRNCPY (aBuff, STRING(PRICE), 20)
                return aBuff;

            case 2:
                SNPRINTF (aBuff, 30, "%.2f", s->GetHigh());
                return aBuff;

            case 3:
                SNPRINTF (aBuff, 30, "%.2f", s->GetLow());
                return aBuff;

            case 4:
                SNPRINTF (aBuff, 30, "%.2f", s->GetClose());
                return aBuff;

            case 5:
                SNPRINTF (aBuff, 30, "%.2f", s->GetOpen());
                return aBuff;

            case 6:
                SNPRINTF (aBuff, 30, "%.0f", s->GetVolume());
                return aBuff;

            default:
                return "";
        }
    }
}



/**
*
*/
int PriceData::width (int col) {
    return XTrader::FontSize() * 6;
}
